package jp.digick.physicom.AdkApp;

import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import com.android.future.usb.UsbAccessory;
import com.android.future.usb.UsbManager;

import android.app.Activity;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.os.ParcelFileDescriptor;
import android.util.Log;

public class AdkAppActivity extends Activity implements Runnable {
	
	private static final String TAG = "ADKAPP";

	private static final String ACTION_USB_PERMISSION = "jp.digik.physicom.AdkApp.action.USB_PERMISSION";

	private static final int MESSAGE_JOYSTICK_XY = 1;
	private static final int MESSAGE_JOYSTICK_BUTTON = 2;
	private static final int MESSAGE_BUTTON = 3;

	private UsbManager mUsbManager;

	private PendingIntent mPermissionIntent;

	private boolean mPermissionRequestPending;

	private UsbAccessory mAccessory;

	private ParcelFileDescriptor mFileDescriptor;
	
	private FileInputStream mInputStream;
	
	private FileOutputStream mOutputStream;

	private JoystickView joystickView;
	private ButtonView buttonView;

	protected class JoyXYMsg {
		private int x;
		private int y;

		public JoyXYMsg(int x, int y) {
			this.x = x;
			this.y = y;
		}

		public int getX() {
			return x;
		}

		public int getY() {
			return y;
		}
	}

	protected class JoyBtnMsg {
		private byte state;

		public JoyBtnMsg(byte state) {
			this.state = state;
		}

		public byte getState() {
			return state;
		}
	}

	protected class ButtonMsg {
		private byte index;
		private byte state;

		public ButtonMsg(byte index, byte state) {
			this.index = index;
			this.state = state;
		}

		public byte getIndex() {
			return index;
		}

		public byte getState() {
			return state;
		}
	}

	private final BroadcastReceiver mUsbReceiver = new BroadcastReceiver() {

		@Override
		public void onReceive(Context context, Intent intent) {
			// TODO Auto-generated method stub
			Log.i(TAG, "onReceive");
			String action = intent.getAction();
			if (ACTION_USB_PERMISSION.equals(action)) {
				synchronized (this) {
					UsbAccessory accessory = UsbManager.getAccessory(intent);
					if (intent.getBooleanExtra(UsbManager.EXTRA_PERMISSION_GRANTED, false)) {
						openAccessory(accessory);
					} else {
						Log.d(TAG, "permission denied for accessory " + accessory);
					}
					mPermissionRequestPending = false;
				}
			} else if (UsbManager.ACTION_USB_ACCESSORY_DETACHED.equals(action)) {
				UsbAccessory accessory = UsbManager.getAccessory(intent);
				if (accessory != null && accessory.equals(mAccessory)) {
					closeAccessory();
				}
			}
		}
		
	};
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Log.i(TAG, "onCreate");
        mUsbManager = UsbManager.getInstance(this);
        mPermissionIntent = PendingIntent.getBroadcast(this, 0, new Intent(ACTION_USB_PERMISSION), 0);
        IntentFilter filter = new IntentFilter(ACTION_USB_PERMISSION);
        filter.addAction(UsbManager.ACTION_USB_ACCESSORY_DETACHED);
        registerReceiver(mUsbReceiver, filter);
        if (getLastNonConfigurationInstance() != null) {
        	mAccessory = (UsbAccessory)getLastNonConfigurationInstance();
        	openAccessory(mAccessory);
        }
        setContentView(R.layout.main);
        joystickView = (JoystickView)findViewById(R.id.joystickView1);
        buttonView = (ButtonView)findViewById(R.id.buttonView1);
    }

    @Override
    public Object onRetainNonConfigurationInstance() {
    	Log.i(TAG, "onRetainNonConfigurationInstance");
    	if (mAccessory != null) {
    		return mAccessory;
    	} else {
    		return super.onRetainNonConfigurationInstance();
    	}
    }

    @Override
    public void onResume() {
    	super.onResume();
    	Log.i(TAG, "onResume");
    	if (mInputStream != null && mOutputStream != null) {
    		return;
    	}
    	
    	UsbAccessory[] accessories = mUsbManager.getAccessoryList();
    	UsbAccessory accessory = (accessories == null ? null : accessories[0]);
    	if (accessory != null) {
    		if (mUsbManager.hasPermission(accessory)) {
    			openAccessory(accessory);
    		} else {
    			synchronized (mUsbReceiver) {
    				if (mPermissionRequestPending) {
    					mUsbManager.requestPermission(accessory, mPermissionIntent);
    					mPermissionRequestPending = true;
    				}
    			}
    		}
    	} else {
    		Log.d(TAG, "mAccessory is null");
    	}
    }
    
    @Override
    public void onPause() {
    	super.onPause();
    	Log.i(TAG, "onPause");
    	closeAccessory();
    }
    
    @Override
    public void onDestroy() {
    	Log.i(TAG, "onDestroy");
    	unregisterReceiver(mUsbReceiver);
    	super.onDestroy();
    }
    
    private void openAccessory(UsbAccessory accessory) {
    	Log.i(TAG, "openAccessory");
    	mFileDescriptor = mUsbManager.openAccessory(accessory);
    	if (mFileDescriptor != null) {
    		mAccessory = accessory;
    		FileDescriptor fd = mFileDescriptor.getFileDescriptor();
    		mInputStream = new FileInputStream(fd);
    		mOutputStream = new FileOutputStream(fd);
    		Thread thread = new Thread(null, this, "AdkApp");
    		thread.start();
    		Log.d(TAG, "accessory opened");
    	} else {
    		Log.d(TAG, "accessory open fail");
    	}
    }
    
    private void closeAccessory() {
    	Log.i(TAG, "closeAccessory");
    	try {
    		if (mFileDescriptor != null) {
    			mFileDescriptor.close();
    		}
    	} catch (IOException e) {
    	} finally {
    		mFileDescriptor = null;
    		mAccessory = null;
    	}
    }
    
	@Override
	public void run() {
		// TODO Auto-generated method stub
		Log.i(TAG, "run");
		int ret = 0;
		byte[] buffer = new byte[16384];
		
		while (ret >= 0) {
			try {
				ret = mInputStream.read(buffer);
			} catch (IOException e) {
				break;
			}
			
			int i = 0;
			while (i < ret) {
				int len = ret - i;
				
				switch (buffer[i]) {
				case 0x01:
					if (len >= 3) {
						Message m = Message.obtain(mHandler, MESSAGE_JOYSTICK_XY);
						m.obj = new JoyXYMsg((int)((buffer[i+1] & 0xff) - 128), (int)(-(buffer[i+2] & 0xff) + 127));
						mHandler.sendMessage(m);
					}
					i += 3;
					break;
				case 0x02:
					if (len >= 3) {
						Message m = Message.obtain(mHandler, MESSAGE_JOYSTICK_BUTTON);
						m.obj = new JoyBtnMsg(buffer[i+2]);
						mHandler.sendMessage(m);
					}
					i += 3;
					break;
				case 0x03:
					if (len >= 3) {
						Message m = Message.obtain(mHandler, MESSAGE_BUTTON);
						m.obj = new ButtonMsg(buffer[i+1], buffer[i+2]);
						mHandler.sendMessage(m);
					}
					i += 3;
					break;
				}
			}
		}
	}
	
	Handler mHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case MESSAGE_JOYSTICK_XY: {
					JoyXYMsg m = (JoyXYMsg)msg.obj;
					joystickView.setPosition(m.getX(), m.getY());
				}
				break;
			case MESSAGE_JOYSTICK_BUTTON: {
					JoyBtnMsg m = (JoyBtnMsg)msg.obj;
					joystickView.setPressed((m.getState() == 0) ? false : true);
				}
				break;
			case MESSAGE_BUTTON: {
					ButtonMsg m = (ButtonMsg)msg.obj;
					switch (m.getIndex()) {
					case 0:
						buttonView.setButton0State(m.getState() != 0);
						break;
					case 1:
						buttonView.setButton1State(m.getState() != 0);
						break;
					case 2:
						buttonView.setButton2State(m.getState() != 0);
						break;
					case 3:
						buttonView.setButton3State(m.getState() != 0);
						break;
					}
				}
				break;
			}
		}
	};
}