package jp.digick.physicom.AdkApp;

import jp.digick.physicom.AdkApp.R;
import jp.digick.physicom.AdkApp.Utilities;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.view.View;

public class JoystickView extends View {
	private Drawable mBackground;
	private Drawable mPressed;
	private Drawable mNormal;

	private int fX;
	private int fY;
	private boolean fPressed;

	private Paint mLabelPaint;
	private String mLabelText;

	public JoystickView(Context context) {
		super(context);
		initJoystickView(context);
	}

	public JoystickView(Context context, AttributeSet attrs) {
		super(context, attrs);
		initJoystickView(context);
	}
	
	private void initJoystickView(Context context) {
		Resources r = context.getResources();
		mBackground = r.getDrawable(R.drawable.joystick_background);
		int w = mBackground.getIntrinsicWidth();
		int h = mBackground.getIntrinsicHeight();
		mBackground.setBounds(0, 0, w, h);
		mPressed = r.getDrawable(R.drawable.joystick_pressed_holo_dark);
		mNormal = r.getDrawable(R.drawable.joystick_normal_holo_dark);
		Utilities.centerAround(w / 2 - 4, h / 2 + 4, mNormal);
		Utilities.centerAround(w / 2 - 4, h / 2 + 4, mPressed);
		mLabelPaint = new Paint();
		mLabelPaint.setColor(Color.WHITE);
		mLabelPaint.setTextSize(24);
		mLabelPaint.setAntiAlias(true);
		mLabelPaint.setShadowLayer(1, 2, 2, Color.BLACK);
		fX = 0;
		fY = 0;
		fPressed = false;
	}
	
	public void setPosition(int x, int y) {
		fX = x - 1;
		fY = y + 3;
		mLabelText = String.format("%d,%d", fX, fY);
		invalidate();
	}
	
	public void setPressed(boolean pressed) {
		fPressed = pressed;
		invalidate();
	}
	
	@Override
	protected void onDraw(Canvas canvas) {
		mBackground.draw(canvas);
		Drawable indicator = fPressed ? mPressed : mNormal;
		int w = mBackground.getIntrinsicWidth();
		int h = mBackground.getIntrinsicHeight();
		int x = w / 2 - 4 + fX;
		int y = h / 2 + 4 + fY;
		Utilities.centerAround(x, y, indicator);
		indicator.draw(canvas);
		canvas.drawText(mLabelText, x + 12, y + 8, mLabelPaint);
	}

	@Override
	protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
		setMeasuredDimension(mBackground.getIntrinsicWidth(),
				mBackground.getIntrinsicHeight());
	}
}
